// This code example is created for educational purpose
// by Thorsten Thormaehlen (contact: www.thormae.de).
// It is distributed without any warranty.

package de.unimarburg.pausedactivityapp;

import android.app.Activity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.LinearLayout;
import android.widget.EditText;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.content.SharedPreferences;

public class PausedActivity extends Activity {

  private int counter = 0; // this value should be maintained per instance
  private int minDigits = 1; // this preference should be maintained globally
  TextView tv;
  EditText et;

  private void updateTextView() {
    tv.setText("Click #" + String.format("%0" + minDigits + "d", counter));
  }

  private OnClickListener cl = new OnClickListener() {
    @Override
    public void onClick(View v) {
      counter++;
      updateTextView();
    }
  };

  private TextWatcher tw = new TextWatcher() {
    @Override
    public void afterTextChanged(Editable s) {
      try {
        minDigits = Integer.parseInt(s.toString());
      } catch (NumberFormatException e) {
        minDigits = 1;
      }
      if (minDigits > 255) {
        minDigits = 255;
      }
      if (minDigits < 1) {
        minDigits = 1;
      }
      updateTextView();
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count,
        int after) {
    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
    }
  };

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    LinearLayout layout = new LinearLayout(this);
    layout.setOrientation(LinearLayout.VERTICAL);
    tv = new TextView(this);
    tv.setText("Hello World");
    layout.addView(tv);

    Button b = new Button(this);
    b.setText("Increment");
    b.setOnClickListener(cl);
    layout.addView(b);
    LinearLayout.LayoutParams lp;
    lp = (LinearLayout.LayoutParams) b.getLayoutParams();
    lp.width = LinearLayout.LayoutParams.WRAP_CONTENT;

    TextView label = new TextView(this);
    label.setText("Choose your preferred minimal number of digits:");
    layout.addView(label);
    et = new EditText(this);
    et.setInputType(InputType.TYPE_CLASS_NUMBER);
    et.setText("1");
    et.addTextChangedListener(tw);
    layout.addView(et);
    lp = (LinearLayout.LayoutParams) et.getLayoutParams();
    lp.width = LinearLayout.LayoutParams.WRAP_CONTENT;
    
    // if available, restore minDigits from preferences
    SharedPreferences prefs = getPreferences(MODE_PRIVATE);
    if (prefs.contains("minDigitsVal")) {
      minDigits = prefs.getInt("minDigitsVal", 1);
      et.setText(Integer.toString(minDigits));
    }

    // if available, restore counter from the savedInstanceState
    if (savedInstanceState != null
        && savedInstanceState.containsKey("counterVal")) {
      counter = savedInstanceState.getInt("counterVal");
      updateTextView();
    } else {
      tv.setText("Hello World");
    }

    this.setContentView(layout);
  }

  @Override
  protected void onPause() {
    super.onPause();
    // write persistent data to preferences
    SharedPreferences prefs = getPreferences(MODE_PRIVATE);
    SharedPreferences.Editor e = prefs.edit();
    e.putInt("minDigitsVal", minDigits);
    e.commit();
  }

  @Override
  public void onSaveInstanceState(Bundle savedInstanceState) {
    // the superclass handles the state of the view classes in the layout
    super.onSaveInstanceState(savedInstanceState);
    // store member variables of this instance to the savedInstanceState
    savedInstanceState.putInt("counterVal", counter);
  }

  private final int ID_MENU_INC = 1;
  private final int ID_MENU_RST = 2;

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    menu.add(Menu.NONE, ID_MENU_INC, Menu.NONE, "Increment");
    menu.add(Menu.NONE, ID_MENU_RST, Menu.NONE, "Reset");
    return true;
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    counter++;
    if (item.getItemId() == ID_MENU_RST) {
      counter = 0;
      minDigits = 1;
      et.setText("1");
    }
    updateTextView();
    return true;
  }
}
